#!/usr/bin/env python
# encoding: utf-8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : argv.py
#   Created Time  : 2019-09-19 19:35
#   Last Modified : 2019-09-19 19:47
#   Describe      :
#
# ====================================================
"""
core: set the numbers of processes
"""
import sys
opt = []
argv = []
core = 200
tmpFile = ['tmp']
#for i in sys.argv[1:]:
#    if i[0] == '-':
#        opt.append(i)
#    else:
#        argv.append(i)



#!/usr/bin/env python
# encoding: utf-8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : trashInfo.py
#   Created Time  : 2019-09-19 15:05
#   Last Modified : 2019-09-19 19:46
#   Describe      :
#
# ====================================================
import sys
import os
from linuxrecycle import argv
from linuxrecycle.config import local_config
from linuxrecycle.config import user
system_dir = ["etc", "bin", "dev", "lib", "lib32", "libx32", "lost+found", "usr", "mnt", "srv", "var"]


def getTrashAddress(afileName):
    """
    Warning: the file name can't contain ~, or $USER,
    please expand them
    1. split the name by $USER
    2. [0]/$USER/.trash
    3. begin with 'etc', 'src'
    """
    afileName = os.path.abspath(afileName)
    # print(afileName)
    file_head = afileName[1:].split('/')[0]
    # print('head', file_head)

    # case 3
    if file_head in system_dir:
        return local_config['core']['default_trash']
    elif file_head == 'media':
        ll = afileName[1:].split('/')[:3]
        # pint(ll)
        ll.append('.trash')
        return os.path.join(ll)

    if user in afileName:
        pp = afileName.split(user)
        return os.path.join(pp[0], user, '.trash')
    trs = "/" + afileName.split("/")[1] + '/.trash'
    return trs


def inTrash(afileName):
    """
    True if the afile is within a .trash
    False not
    Once found ${USER}/.trash in the path, then the path is determined within
    a trash
    """
    tak = '/.trash/'
    if tak in afileName:
        return True
    return False


def tmpFile(afileName):
    """
    True if the file is fileInfo.db
    """
    tak = afileName.split("/")[1]
    if afileName in ['tmp']:
        return True
    return False


def whiteFile(afileName):
    """
    True if the in /tmp/...
    """
    tak = afileName.split("/")[1]
    if tak in local_config['core']['white_files'].split(','):
        return True
    return False


if __name__ == "__main__":
    testf = os.environ["PWD"]
    print((getTrashAddress(testf)))
    ff = getTrashAddress(testf) + "/dada/dada"
    if inTrash(ff):
        print(ff)
        print("In trash")



#!/usr/bin/env python
# encoding: utf-8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : printDB.py
#   Created Time  : 2019-09-27 14:37
#   Last Modified : 2019-10-07 13:27
#   Describe      :
#
# ====================================================
import time
import sys
from termcolor import colored
from linuxrecycle import DB


def init():
    """
    show help 
    """
    if '-h' in sys.argv[1:]:
        print("usage : show the last #n records") 
        print("    rm.printDB n") 
        exit(0)
    return

def main():
    '''
    function:
        print the last #n record information about the files deleted
    '''
    init()
    n = 100
    try:
        n = int(sys.argv[1])
    except IndexError as e:
        pass
    except ValueError as e:
        print("Error:: Please input a integer!!! .> -> {}".format(sys.argv[1]))
        raise e
    for inf in sorted(DB.getLastRecord(n, "exits='exits'"), key=lambda x:x[0]):
        # print(inf)
        date = time.asctime(time.localtime(inf[4])).replace("Sat", "@")
        print (" ".join([colored(inf[0], 'blue','on_grey', attrs=['bold']), colored(date,'green'
                ), inf[1]]))
if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# encoding: utf-8
# coding style: pep8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin Ma
#   Email         : xxmawhu@163.com
#   File Name     : setCrontab.py
#   Last Modified : 2019-11-05 10:41
#   Describe      :
#
# ====================================================

# import sys
import os
import subprocess as sp
from linuxrecycle import config


class AutoRegularTask:
    """
    regular execute a command by `crontab`
    """

    def __init__(self):
        self._newCommand = set()
        self._oldCommand = ''
        self._tmpFile = []

    def Add(self, command, hour="3", minute="14", month="*", day="*", week="*"):
        new_command = "{Minute} {Hour} {Day} {Month} {Week} {Command}".format(
            Minute=minute, Hour=hour, Command=command, Month=month, Day=day, Week=week
        )
        self._newCommand.add(new_command)

    def _getOldCrontab(self):
        """
        return crontab -l
        """
        process = sp.Popen(['crontab', '-l'], stdout=sp.PIPE, stderr=sp.PIPE)
        out, err = process.communicate()
        self._oldCommand = out.decode()

    def setAllCommands(self):
        """
        add the commands on one by one
        """
        self._getOldCrontab()
        f = open("tmp.txt", 'w')
        f.write(self._oldCommand)
        for new_command in self._newCommand:
            if new_command in self._oldCommand:
                continue
            f.write("{}\n".format(new_command))
        f.close()
        self._tmpFile.append("tmp.txt")
        process = sp.Popen(['crontab', '-u', config.user, "tmp.txt"], stdout=sp.PIPE, stderr=sp.PIPE)
        out, err = process.communicate()

    def removeTmpFile(self):
        """
        As its name, remove all tmp file such as "tmp.txt"
        """
        for f in self._tmpFile:
            try:
                os.remove(f)
            except Exception as e:
                raise e
            finally:
                pass

    def setup(self):
        self.setAllCommands()
        self.removeTmpFile()
        print("setup sucessful!")
        self._getOldCrontab()
        print(self._oldCommand)



#!/usr/bin/env python
# encoding: utf-8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : DB.py
#   Created Time  : 2019-09-19 16:31
#   Last Modified : 2019-09-27 14:45
#   Describe      :
#
# ====================================================
"""usage: DB.py
some functions to operate the data base
1.insertDB(var type: list of tuple)
  insert some information to the data base
"""

import sqlite3
from sqlite3 import OperationalError
import os
# import time
from linuxrecycle.config import local_config
query = """
CREATE TABLE IF NOT EXISTS fileInfo
(id INTEGER PRIMARY KEY, 
 address text, 
 trashAddress text, type text,
time real, date text, exits text)
"""

# @profile
def initDB():
    conn = sqlite3.connect(local_config.get('core', 'data_base_file'))
    try:
        cursor = conn.cursor()
        cursor.execute(query)
        cursor.execute("CREATE index  fileInfoIndex on fileInfo(id) ")
        conn.commit()
    # except Exception:
    #    pass
    finally:
        conn.close()
    return


# @profile
def lastID():
    conn = sqlite3.connect(local_config.get('core', 'data_base_file'), timeout=30.0)
    ID = 0
    try:
        cursor = conn.cursor()
        cursor.execute("SELECT * FROM fileInfo ORDER BY id DESC limit 1")
        rows = cursor.fetchall()
        if rows:
            ID = rows[0][0]
    finally:
        cursor.close()
        conn.close()
    return ID


def insertDB(information):
    # add ID
    ID = lastID() + 1
    # print ID
    IDinfo = []
    for indx, item in enumerate(information):
        # print "id:", indx, "item",  item
        IDinfo.append((ID + indx, ) + item)
    conn = sqlite3.connect(local_config.get('core', 'data_base_file') )
    insertmt = 'INSERT INTO fileInfo VALUES(?, ?, ?, ?, ?, ?, ?)'
    # print IDinfo
    cursor = conn.cursor()
    cursor.execute("begin transaction")
    try:
        cursor.executemany(insertmt, IDinfo)
    except OperationalError as e:
        if "database is locked" in e:
            print("fun<insertDB>", "OperationalError :", e)
            insertDB(information)
        if "no such table" in e:
            print("OperationalError :", e)
            cursor.execute(query)
            cursor.executemany(insertmt, IDinfo)
            conn.commit()
    # except Exception as e:
    #    print "fun<insertDB>", e
    finally:
        conn.commit()
        cursor.close()
        conn.close()


def getAllInf():
    """
    get all information from the data base
    """
    con = sqlite3.connect(local_config.get('core', 'data_base_file'),
                          timeout=30.0)
    cursor = con.cursor()
    cursor = con.execute("SELECT * FROM fileInfo")
    rows = cursor.fetchall()
    return rows


def getLastRecord(n, condition=''):
    """
    get the record of last #n
    """
    con = sqlite3.connect(local_config.get('core', 'data_base_file'),
                          timeout=30.0)
    cursor = con.cursor()
    sel_commond = "SELECT * FROM fileInfo"
    if condition:
        sel_commond += " WHERE {} ".format(condition)
    sel_commond += "ORDER BY id DESC limit {}".format(n)
    # print("condition {}".format(condition))
    # print("execute {}".format(sel_commond))
    cursor.execute(sel_commond)
    rows = cursor.fetchall()
    cursor.close()
    con.close()
    return rows


def get_record_by_id(ID):
    """
    get the record with certain ID 
    """
    con = sqlite3.connect(local_config.get('core', 'data_base_file'))
    try:
        cursor = con.cursor()
        cursor.execute("SELECT * FROM fileInfo WHERE id = {}".format(ID))
        rows = cursor.fetchall()
    finally:
        cursor.close()
    con.close()
    return rows


def clearDB():
    con = sqlite3.connect(local_config.get('core', 'data_base_file'),
                          timeout=30.0)
    delQuery = "DELETE from fileInfo"
    try:
        cursor = con.cursor()
        cursor.execute(delQuery)
        con.commit()
    # except Exception as e:
    #     print "fun<clearDB>", e
    finally:
        con.close()


def delByID(Id):
    """
    Args:
        Id: the Index the the one creod 
    return:
        void
    """
    con = sqlite3.connect(local_config.get('core', 'data_base_file'),
                          timeout=30.0)
    try:
        cursor = con.cursor()
        delQuery = "DELETE from fileInfo WHERE id=?"
        cursor.execute(delQuery, (Id, ))
        con.commit()
    # except Exception as e:
    #    print "fun<delByID>", e
    finally:
        con.close()


# initi the fileInfo.config.db
if not os.path.isfile(local_config.get('core', 'data_base_file')):
    initDB()
if __name__ == "__main__":
    if not os.path.isfile(local_config.get('core', 'data_base_file')):
        initDB()
    print(lastID())
    exit()

    for inf in getAllInf():
        print(inf)
    print(lastID())
    print(get_record_by_id(1))



#!/usr/bin/env python
# encoding: utf-8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : main.py
#   Created Time  : 2019-09-19 19:46
#   Last Modified : 2019-10-07 11:08
#   Describe      :
#
# ====================================================
import glob
import time
import sys
from linuxrecycle.config import local_config
from linuxrecycle import DB
from linuxrecycle import oper
rmForce = False
if '-f' in sys.argv:
    rmForce = True


def Rm(afile):
    # rm one file for use of pool.map
    # rm_status is the status of oprater 'mv' or 'rm'
    rm_status = ()
    if rmForce:
        rm_status = oper.RmForce(afile)
    else:
        rm_status = oper.MoveToTrash(afile)
    return rm_status


def obtainAllFile():
    files = []
    for i in sys.argv[1:]:
        files += glob.glob(i)
    return files


def test():
    executor = Pool(local_config.getint('core', 'Num_Processor'))
    logs = executor.map(Rm, ['LinuxRecycle.egg-info', 'build'])
    executor.close()
    executor.join()
    DB.insertDB(logs)


def main():
    if rmForce:
        logs = list(map(oper.RmForce, obtainAllFile()))
    else:
        logs = list(map(oper.MoveToTrash, obtainAllFile()))
    logCol = []
    for log in logs:
        if log[-1] == "exits":
            logCol.append(log)
    DB.insertDB(logCol)


if __name__ == "__main__":
    main()
    exit(0)



#!/usr/bin/env python3
# encoding: utf-8
# coding style: pep8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin Ma
#   Email         : xxmawhu@163.com
#   File Name     : setCrontab.py
#   Last Modified : 2019-11-05 11:26
#   Describe      :
#
# ====================================================

# import sys
import os
import subprocess as sp
from AutoRegularTask import AutoRegularTask
if __name__ == "__main__":
    crontab = AutoRegularTask()
    crontab.Add("python {}".format(os.path.abspath("auto_clear.py")),
            hour = "4")
    crontab.setup()



#!/usr/bin/env python
# encoding: utf-8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : auto_clear.py
#   Created Time  : 2019-09-27 20:05
#   Last Modified : 2019-10-08 09:54
#   Describe      :
#
# ====================================================
import time
import sqlite3
import shutil
import os
from linuxrecycle import config


def delete_old_file(raws):
    """
    delete the file in the .trash
    info[2]
    """
    if raws:
        print("remove :")
    for info in raws:
        file_name = info[2]
        print(file_name)
        if info[3] != 'd':
            try:
                os.remove(file_name)
            except OSError:
                pass
        else:
            try:
                shutil.rmtree(file_name)
            except OSError:
                pass
    return


def update(rows):
    """
    func: update the record information to 'removed'
    """
    update_state = 'UPDATE fileInfo SET exits="removed" WHERE id==?'
    conn = sqlite3.connect(config.local_config.get('core', 'data_base_file'))
    try:
        cursor = conn.cursor()
        # print([int(i[0]) for i in rows])
        cursor.executemany(update_state, [(str(i[0]),) for i in rows])
        # rows = cursor.fetchall()
        conn.commit()
        cursor.close()
    finally:
        conn.close()
    return


def main():
    """
    1. select the record with time less than t_0 and exits = ""exits
    2. delete them and update the data base
    """
    conn = sqlite3.connect(config.local_config.get('core', 'data_base_file'))
    t0 = time.time() - 3600.0 * 24.0 * config.local_config.getint('core', 'keep_days')
    print("delete files more than {} days".format(
        config.local_config.get('core', 'keep_days')))
    # t0 = time.time() - 3600.0 * 24.0 * 1.0  
    # config.local_config.getint('core', 'keep_days')
    query = "SELECT * FROM fileInfo "
    query += ' WHERE  time < {} AND exits!="removed"'.format(t0)
    raws = []
    try:
        cursor = conn.cursor()
        cursor.execute(query)
        raws = cursor.fetchall()
    finally:
        cursor.close()
        conn.close()
    # print rows
    if not raws:
        print("[Info] no files are needed to be removed!")
    delete_old_file(raws)
    update(raws)
    # write log
    log_file = os.path.expanduser('~/.mtdb/log')
    f = open(log_file, 'a')
    date = time.strftime("%Y-%m-%d:%H:%M:%S", time.localtime())
    f.write('auto clear trash @ {}\n'.format(date))
    f.close()
    return


if __name__ == "__main__":
    main()



#!/usr/bin/env python3
# encoding: utf-8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : config.py
#   Created Time  : 2019-09-19 23:52
#   Last Modified : 2019-09-26 16:00
#   Describe      :
#
# ====================================================
import os
from configparser import ConfigParser
local_config = ConfigParser()
if os.path.exists(os.path.expanduser('~/.mtdb/mtrc')):
    local_config.read(os.path.expanduser('~/.mtdb/mtrc'))
else:
    mtdbDir = os.path.expanduser('~/.mtdb')
    if not os.path.exists(mtdbDir):
        os.mkdir(mtdbDir)
    local_config.add_section('core')
    local_config.set('core', 'Num_Processor', "100")
    local_config.set('core', 'Keep_data_base', "1")
    local_config.set('core', 'keep_days', "30")
    local_config.set('core', "default_trash", os.path.expanduser("~/.trash"))
    local_config.set('core', 'data_base_file', os.path.expanduser("~/.mtdb/fileInfo.db"))
    local_config.set('core', 'white_files', os.path.expanduser("~/.mtdb/fileInfo.db") + ',')
    with open(os.path.expanduser('~/.mtdb/mtrc'), 'w') as configFile:
        local_config.write(configFile)
user = ''
if "USER" in os.environ:
    user = os.environ["USER"]
elif "LOGNAME" in os.environ:
    user = os.environ['LOGNAME']
elif 'USERNAME' in os.environ:
    user = os.environ['USERNAME']
else:
    print("Error, can not find USER, LOGNAME, or USERNAME")


def makeAutoClear():
    mtdbDir = os.path.expanduser('~/.mtdb')
    f = open(mtdbDir + "/clear.py", 'w')
    f.write("#!/usr/bin/env python3\n")
    f.write("import linuxrecycle.auto_clear as cl\n")
    f.write("cl.main()\n")
    f.close()


autoClear = os.path.expanduser('~/.mtdb/clear.py')
if not os.path.exists(autoClear):
    makeAutoClear()

if __name__ == "__main__":
    print(local_config['core']['Num_Processor'])
    print(local_config['core']['keep_days'])
    print(local_config['core']['data_base_file'])
    print(local_config['core']['default_trash'])
    print("USER {}".format(user))
# print local_config.get('core', 'white_file')



#!/usr/bin/env python
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : setup.py
#   Created Time  : 2019-09-24 10:51
#   Last Modified : 2019-09-29 12:09
#   Describe      :
#
# ====================================================

from setuptools import setup
from setuptools import find_packages
import sys
import os

m_version = '1.9.2'

if sys.argv[1] == "publish":
    os.system("python3 setup.py sdist")
    os.system("python3 setup.py bdist_wheel")
    os.system("twine upload dist/*{}*".format(m_version))
else:
    setup(
        name='LinuxRecycle',
        version=m_version,
        author='Xin-Xin Ma',
        description="A recycle system for linux",
        long_description=open("./README.md").read(),
        packages=find_packages(),
        data_files=[("", ["LICENSE"])],
        license="GPL",
        project_urls={
            'Source': 'https://github.com/xxmawhu/LinuxRecycle',
        },
        entry_points={
            'console_scripts':
                [
                    'rm=linuxrecycle.main:main', 'rm.printDB=linuxrecycle.printDB:main',
                    'rm.clean=linuxrecycle.auto_clear:main', 'rm.recover=linuxrecycle.recover:main'
                ]
        },
        install_requires=[
            'termcolor',
        ]
    )



#!/usr/bin/env python
# encoding: utf-8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : printDB.py
#   Created Time  : 2019-09-27 14:37
#   Last Modified : 2019-09-27 14:41
#   Describe      :
#
# ====================================================

from linuxrecycle import DB
import time
import sys
from termcolor import colored
import shutil

def recover(ID):
    """
    recover one file by the ID
    """
    inf = DB.get_record_by_id(ID)
    if not inf:
        print("Error: voild ID {}".format(ID))
        return
    record_inf = inf[0]
    # print record_inf
    try:
        shutil.copy2(record_inf[2], record_inf[1])
    except IsADirectoryError:
        shutil.copytree(record_inf[2], record_inf[1])
    finally:
        pass

def init():
    """
    show the help
    """
    if '-h'  in sys.argv[1:]:
        print("usage: recover id1 [id2 id3]")
        exit(0)
    return

def main():
    '''
    usage:
       recover id1 [id2 id3]
    '''
    init()
    for ID in list(map(int, sys.argv[1:])):
        recover(ID)
if __name__ == "__main__":
    main()



#!/usr/bin/env python
# encoding: utf-8
# ====================================================
#   Copyright (C)2019 All rights reserved.
#
#   Author        : Xin-Xin MA
#   Email         : xxmawhu@163.com
#   File Name     : oper.py
#   Created Time  : 2019-09-19 15:05
#   Last Modified : 2019-10-07 11:10
#   Describe      :
#
# ====================================================
import os
import time
from linuxrecycle import trashInfo
import subprocess as sp
import shutil
import glob


def RmForce(fileName):
    """
    if -f in opt, rm the file directly
    """
    address = os.path.abspath(fileName)
    exits = "removed"
    Type = 'f'
    date = time.strftime("%Y-%m-%d:%H:%M:%S", time.localtime())
    try:
        os.remove(address)
    except OSError as e:
        # [Error 21] Is directory: ...
        if e.args[0] == 21:
            Type = 'd'
            shutil.rmtree(address)
        else:
            raise e
        pass
    finally:
        return (address, "", Type, time.time(), date, exits)


#@profile
def MoveToTrash(address):
    """
    Warning: the file must be an exact file or directory. The following cases
    are not allowed
    jobs/*,  *.py, jobs_00?.txt, */*.py
    staus will be returned, like
    (address, trashAddress, type, time, date, exits)
    """
    address = os.path.abspath(address)
    # print("address :", address)
    trashName = trashInfo.getTrashAddress(address)
    # print("trash name: " + trashName)
    exits = "exits"
    Type = 'f'
    if os.path.isdir(address):
        Type = 'd'
    date = time.strftime("%y%m%d_%H%M%S", time.localtime())
    if trashInfo.inTrash(address) or trashInfo.tmpFile(address):
        exits = "removed"
        RmForce(address)
        return (address, "", Type, time.time(), date, exits)
    if trashInfo.whiteFile(address):
        return (address, address, Type, time.time(), date, exits)
    else:
        trashAddress = os.path.join(trashName, address.split("/")[-1] + "." + date)
        # print("Address in trash : ", trashAddress)
        # print("mv  {} {}".format(address, trashAddress))
        try:
            shutil.move(address, trashAddress)
            return (address, trashAddress, Type, time.time(), date, exits)
        except PermissionError as e:
            print(e)
            if e.errno == 2:
                print(e)
            return (address, address, Type, time.time(), date, exits)
        except IOError as e:
            print(e)
            os.mkdir(trashName)
            shutil.move(address, trashAddress)
            return (address, trashAddress, Type, time.time(), date, exits)
        except OSError as e:
            print(e)
            return (address, address, Type, time.time(), date, exits)

    return ()


if __name__ == "__main__":
    testf = os.environ["PWD"] + "/dada.txt"
    os.system("touch {}".format(testf))
    RmForce(testf)
    exit()
    os.system("touch {}".format(testf))
    print((MoveToTrash(testf)))
    for ff in glob.glob("dada*"):
        # print ff
        MoveToTrash(ff)
